﻿/**
 * Scroller
 * User: lihao.ylh/65936
 * Date: 2014/1/13
 * Time: 下午6:46
 */
define('lofty/ui/scroller/1.0/scroller', ['lofty/ui/scroller/1.0/iscroll-base', 'lofty/lang/class', 'lofty/lang/base', 'lofty/gallery/appframework/af'],
	function(iScrollBase, Class, Base, $){
		var Scroller = Class(Base, {
			init: function(opts) {

				var that = this, pullDownflag = false, pullUpflag = false;
				this.container = opts.container ? opts.container.replace('#', '') : opts.container;

				// 没有指定container直接返回
				if (!this.container) {
					return;
				}
				this.topGapThreshold = opts.topGapThreshold || 5;
				this.topSelector = opts.topSelector;
				this.bottomGapThreshold = opts.bottomGapThreshold || 5;
				this.bottomSelector = opts.bottomSelector;

				if ($(that.topSelector).length) {
					this.topSelectorHeight = $(that.topSelector).outerHeight();
				}

				// 截获构造函数中传入的事件，换成用on的方式来绑定。
				this._bindEvent(opts);
				opts = extend(opts, {
					topOffset: opts.topOffset || that.topSelectorHeight,
					onBeforeScrollStart: function(e) {
						// 开始滚动前的事件回调
						// 默认是阻止浏览器默认行为
						e.preventDefault();
						that.trigger('beforeScrollStart', e);
					},
					onScrollStart: function() {
						that.trigger('scrollStart');
					},
					onBeforeScrollMove: function(e) {
						// 开始滚动前的事件回调
						// 默认是阻止浏览器默认行为
						e.preventDefault();
						that.trigger('beforeScrollMove')
					},
					onScrollMove: function(e) {
						that.trigger('scrollMove', e);

						if (that.topSelector) {
							
							// 判断是否需要触发pullDownChange事件
							if (!pullDownflag && that.iScroller.y > that.topGapThreshold) {
								pullDownflag = true;
								that.trigger('pullDownChange');
								that.iScroller.minScrollY = 0;
							}
							// 判断是否需要触发pullDownRestore事件
							if (pullDownflag && that.iScroller.y <= that.topGapThreshold) {
								//debugger;
								pullDownflag = false;
								that.trigger('pullDownRestore');
								that.iScroller.minScrollY = -that.topSelectorHeight;
							}
						}

						//bottomSelectorHeight = $(that.bottomSelector).outerHeight();
						// 判断是否需要触发pullDownChange事件
						if (!pullUpflag && that.iScroller.y < (that.iScroller.maxScrollY - that.bottomGapThreshold)) {
							pullUpflag = true;
							that.trigger('pullUpChange');
						}
						// 判断是否需要触发pullDownRestore事件
						if (pullUpflag && that.iScroller.y >= (that.iScroller.maxScrollY - that.bottomGapThreshold)) {
							pullUpflag = false;
							that.trigger('pullUpRestore');
						}
					},
					onBeforeScrollEnd: function() {
						that.trigger('beforeScrollEnd');
					},
					onScrollEnd: function() {
						that.trigger('scrollEnd');
						if (pullDownflag) {
							that.trigger('pullToRefresh');
						}
						if (pullUpflag) {
							that.trigger('infiniteScroll');
						}
					},
					onTouchEnd: function() {
						that.trigger('touchEnd');
					},
					onRefresh: function() {
						that.trigger('refresh');
						// 每次调用refresh事件时都需要重置标志位
						pullDownflag = false;
						pullUpflag = false;
					},
					onDestroy: function() {
						that.trigger('destroy');
					},
					onZoomStart: function() {
						that.trigger('zoomStart');
					},
					onZoom: function() {
						that.trigger('zoom');
					},
					onZoomEnd: function() {
						that.trigger('zoomEnd');
					}
				});
				this.iScroller = new iScrollBase(this.container, opts);

				// 代理原iScroll的方法
				this._delegateMethods();
			},
			_bindEvent: function(opts) {
				var oldOnScrollMove = opts.onScrollMove,
					oldOnScrollEnd = opts.onScrollEnd,
					oldOnRefresh = opts.onRefresh;

				if (oldOnScrollMove) {
					this.on('scrollMove', oldOnScrollMove);
				}
				if (oldOnScrollEnd) {
					this.on('scrollEnd', oldOnScrollEnd);
				}
				if (oldOnRefresh) {
					this.on('refresh', oldOnRefresh);
				}
				if (opts.onPullDownChange) {
					this.on('pullDownChange', opts.onPullDownChange);
				}
				if (opts.onPullDownRestore) {
					this.on('pullDownRestore', opts.onPullDownRestore);
				}
				if (opts.onPullUpChange) {
					this.on('pullUpChange', opts.onPullUpChange);
				}
				if (opts.onPullUpRestore) {
					this.on('pullUpRestore', opts.onPullUpRestore);
				}
				if (opts.onPullToRefresh) {
					this.on('pullToRefresh', opts.onPullToRefresh);
				}
				if (opts.onInfiniteScroll) {
					this.on('infiniteScroll', opts.onInfiniteScroll);
				}
			},
			_delegateMethods: function() {
				var delegatedMethods = [
					'destory', 
					'refresh', 
					'scrollTo',
					'scrollToElement',
					'scrollToPage',
					'disable',
					'enable',
					'stop',
					'zoom',
					'isReady'
				];
				for (var i=0,leni=delegatedMethods.length; i<leni; i++) {
					this[delegatedMethods[i]] = (function(i) {
						return function() {
							this.iScroller[delegatedMethods[i]].apply(this.iScroller, arguments);
						}
					})(i)
				}
			},
			get: function(key) {
				return this.iScroller[key];
			},
			set: function(key, value) {
				this.iScroller[key] = value;
			},
			getContainer: function() {
				return '#' + this.container;
			}
		})

		/**
		 * 简易合并函数，将后边的对象合入前边的对象
		 * @param  {[type]} dest [description]
		 * @param  {[type]} src  [description]
		 * @return {[type]}      [description]
		 */
		function extend(dest, src) {
			var ret = {};
			for (var prop in dest) {
				ret[prop] = dest[prop];
			}
			for (var prop in src) {
				ret[prop] = src[prop];
			}
			return ret;
		}

		return Scroller;
	});